﻿--
-- Merlo P41.7 Turbofarmer
-- FS15 Edition
-- 
-- author  Mythos
-- date  11/02/2015
-- www.ARM-team.gr
-- Copyright (C) ARM Team, All Rights Reserved.

MerloP417 = {};

function MerloP417.prerequisitesPresent(specializations)
    return true;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:load(xmlFile)

	-- Functions
	self.MPCode = SpecializationUtil.callSpecializationsFunction("MPCode");
	self.serviceEngine = SpecializationUtil.callSpecializationsFunction("serviceEngine");
	self.findServicePoints = SpecializationUtil.callSpecializationsFunction("findServicePoints");
	self.getServiceDistance = SpecializationUtil.callSpecializationsFunction("getServiceDistance");
	self.playerCallback = SpecializationUtil.callSpecializationsFunction("playerCallback");
	
	-- Indexs
	self.outCamera = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cameras.camera1#rotateNode"));
	self.refPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.refPoint#index"));
	self.refPointS = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.refPointS#index"));
	self.joystick = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.joystick#index"));
	self.coolerFan = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.coolerFan#index"));
	
	self.backAtt = Utils.indexToObject(self.components,getXMLString(xmlFile,"vehicle.backAttacher#index"));
	self.baTransMin = getXMLFloat(xmlFile,"vehicle.backAttacher#transMin");
	self.baTransMax = getXMLFloat(xmlFile,"vehicle.backAttacher#transMax");
	
	-- Triggers
	self.MTriggers = {};
	local i=0;
	while true do
		local key = string.format("vehicle.MTriggers.MTrigger(%d)", i);
		local trigger = Utils.indexToObject(self.components,getXMLString(xmlFile, key.. "#index"));
		if trigger == nil then
			break;
		end;
		self.MTriggers[i+1] = {};
		self.MTriggers[i+1].trigger = trigger;
		self.MTriggers[i+1].enable = false;
		
		addTrigger(trigger, "playerCallback", self);				
		i = i + 1;
	end;
	
	-- Anims
	self.windowAnim = getXMLString(xmlFile, "vehicle.windowAnim#animationName");
	self.doorAnim = getXMLString(xmlFile, "vehicle.doorAnim#animationName");
	self.bonnetAnim = getXMLString(xmlFile, "vehicle.bonnetAnim#animationName");
	
	self.oldY = 0;
	self.diff1 = 0;
	self.changeWheel = 0;
	self.ct = 7000;
	
	-- Service Attributes
	self.nextService = 10;
	self.sumHours = 0;
	self.servicePoints = {};
	self.serviceDistances = {};
	
	-- Booleans
	self.WM = false;
	self.DM = false;
	self.BM = false;
	self.AC = false;
	self.ACC = false;
	self.SC = false;
	self.SCC = false;
	self.alarm = false;
	
	-- Layers
	self.SCx = 0.826;
    self.SCy = 0;
    self.SCWidth = 0.266;
    self.SCHeight = 0.029;
	
	self.sX = 0.847;
    self.sY = 0.009;
	
	-- Alarm Console
	self.ACFile = Utils.getFilename("textures/AlarmConsole.png", self.baseDirectory);
	self.ACOL = Overlay:new("ACOL", self.ACFile, self.SCx, self.SCy, self.SCWidth, self.SCHeight);
	
	-- Service Console
	self.SCFile = Utils.getFilename("textures/ServiceConsole.png", self.baseDirectory);
	self.SCOL = Overlay:new("SCOL", self.SCFile, self.SCx, self.SCy, self.SCWidth, self.SCHeight);
	
	self:findServicePoints(getRootNode(), "VehicleSellingPoint.onCreate");
	
	-- Sounds
	faultSoundFile = Utils.getFilename("Sounds/faultSound.wav", self.baseDirectory);
	self.faultSound = createSample("faultSound");
	loadSample(self.faultSound, faultSoundFile, false);
	
	alarmSoundFile = Utils.getFilename("Sounds/alarm.wav", self.baseDirectory);
	self.alarmSound = createSample("alarmSound");
	loadSample(self.alarmSound, alarmSoundFile, false);
	
	serviceSoundFile = Utils.getFilename("Sounds/service.wav", self.baseDirectory);
	self.serviceSound = createSample("serviceSound");
	loadSample(self.serviceSound, serviceSoundFile, false);
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:delete()
	for i,j in pairs(self.MTriggers) do
		if j.trigger ~= nil then
			removeTrigger(j.trigger);
		end;
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:mouseEvent(posX, posY, isDown, isUp, button)
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:keyEvent(unicode, sym, modifier, isDown)
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:update(dt)
	
	local enable = false;
	local id = 0;
	for i,j in pairs(self.MTriggers) do
		if j.enable then
			id = i;
			enable = true;
			break;
		end;
	end;
	
	if self:getIsActiveForInput() and self:getIsActive() then
		
		if InputBinding.hasEvent(InputBinding.MERLOP417_WHEEL_MODE) then
			self.changeWheel = self.changeWheel+1;
		end;
		
		if self.changeWheel == 0  then
			self:MPCode(2, 0, 0);
		elseif self.changeWheel == 1  then
			self:MPCode(2, 1, -50);
		elseif self.changeWheel == 2  then
			self:MPCode(2, 2, 50);
		elseif self.changeWheel == 3 then
			self.changeWheel = 0;
		end;
	
		if InputBinding.isPressed(InputBinding.MERLO_BACK_ATTACHER_UP) then
			self:MPCode(3);
		end;
		
		if InputBinding.isPressed(InputBinding.MERLO_BACK_ATTACHER_DOWN) then
			self:MPCode(4);
		end;
		
		if InputBinding.hasEvent(InputBinding.MERLO_DOOR_MODE) then
			self:MPCode(6, nil, nil, nil, nil, nil, not self.DM);
		end;
		
		if not self.DM then
			if InputBinding.hasEvent(InputBinding.MERLO_WINDOW_MODE) then
				self:MPCode(5, nil, nil, nil, nil, nil, not self.WM);
			end;
		end;
		
		local arma1 = self.movingTools[1].lastRotSpeed;
		local arma2 = self.movingTools[2].lastTransSpeed;
		local arma3 = self.movingTools[3].lastRotSpeed;
		
		if arma1~=0 then
			self.a1 = math.rad(18)*arma1/0.000977;
		else
			self.a1 = 0;
		end;
		
		if arma2~=0 then
			self.a2 = math.rad(100)*arma2/0.036;
		else
			self.a2 = 0;
		end;
		
		if arma3~=0 then
			self.a3 = math.rad(100)*arma3/0.01675;
		else
			self.a3 = 0;
		end;
		
		if self.a1~=0 or self.a2~=0 or self.a3~=0 then
			self:MPCode(8, nil, nil, self.a1, self.a2, self.a3);
		end;
		
	end;
	
	if self.AC or self.SC then
		if table.getn(self.servicePoints) >0 then
			for i,v in pairs(self.servicePoints) do
				self:getServiceDistance(v);
				self.serviceDistances[i] = self.distance;
			end;
		end;
		for i,v in pairs(self.serviceDistances) do
			if v <=4 then
				if self.BM then
					if not self.isMotorStarted then
						if enable then
							if id==1 then
								g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_SERVICE_MODE"), self.typeDesc), InputBinding.MERLO_SERVICE_MODE);
								if InputBinding.hasEvent(InputBinding.MERLO_SERVICE_MODE) then
									self:serviceEngine();
								end;
							end;
						end;
					end;
				end;
				break;
			end;
		end;
	end;
	
	if enable then
		if id==1 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_BONNET_MODE"), self.typeDesc), InputBinding.MERLO_BONNET_MODE);
			if InputBinding.hasEvent(InputBinding.MERLO_BONNET_MODE) then
				self:MPCode(7, nil, nil, nil, nil, nil, not self.BM);
			end;
		end;
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:getServiceDistance(nodeId)
	local sx, sy, sz = getWorldTranslation(self.rootNode);
	local nx, ny, nz = getWorldTranslation(nodeId);
	self.distance = Utils.vector3Length(sx-nx, sy-ny, sz-nz);
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:findServicePoints(nodeId, name)
	local getAttr = getUserAttribute(nodeId, "onCreate");
	if getAttr == name then
		local servicePoint = getChildAt(nodeId, 1);
		table.insert(self.servicePoints, servicePoint);
	end;
	
	for i=0, getNumOfChildren(nodeId)-1 do
		self:findServicePoints(getChildAt(nodeId, i), name);
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:serviceEngine()
	g_currentMission.missionStats.money = g_currentMission.missionStats.money - 300;
	self.motor.maxRpm = 2200;
	playSample(self.serviceSound, 1, 1, 0);
	self.nextService = self.sumHours + 10;
	self.AC = false;
	self.SC = false;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:onEnter()
	if self.isMotorStarted and self.SC then
		playSample(self.faultSound, 0, 0.6, 0);
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:onLeave()
	stopSample(self.faultSound);
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:updateTick(dt)
	
	if self.isMotorStarted then
		rotate(self.coolerFan, 0, 0, dt*0.005);
		self.sumHours = self.sumHours + 1.0*dt/(1000*3600);
		
		local serDif = self.nextService - self.sumHours;
		
		if serDif<=1 and serDif>0 then
			if not self.ACC then
				playSample(self.alarmSound, 1, 1, 0);
				self.alarm = true;
			end;
			self.AC = true;
			self.ACC = true;
		end;
		
		if serDif<=0 then
			self.motor.maxRpm = 1001;
			if not self.SCC then
				playSample(self.alarmSound, 1, 1, 0);
				playSample(self.faultSound, 0, 0.6, 0);
				self.alarm = true;
			end;
			self.AC = false;
			self.SC = true;
			self.SCC = true;
		end;
	else
		stopSample(self.faultSound);
		self.ACC = false;
		self.SCC = false;
	end;
	
	if self:getIsActive() then
		local x,y,z = getWorldTranslation(self.refPoint);
		local x1,y1,z1 = getWorldTranslation(self.refPointS);
		local diff = y-y1;
		self.diff1 = diff - self.oldY;
		if self.diff1~=0 then
			self:MPCode(1);
		end;
		self.oldY = diff;
		
		if self.alarm then
			self.ct = self.ct - dt;
			if self.ct <0 then
				self.alarm = false;
				self.ct = 7000;
			end;
		end;
		
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:MPCode(mode, state, angle, a1, a2, a3, state2, noEventSend)
	MerloP417Event.sendEvent(self, mode, state, angle, a1, a2, a3, state2, noEventSend);

--______________________________________________________________________________
	if mode == 1 then
--____________________________ ADJUST OUT CAMERA HEIGHT _________________________
		translate(self.outCamera, 0, self.diff1/4, 0);
--______________________________________________________________________________
	elseif mode == 2 then
--___________________________________ WHEEL MODE _______________________________
		self.wheels[3].rotSpeed = Utils.degToRad(angle);
		self.wheels[4].rotSpeed = Utils.degToRad(angle);
		if state == 0  then
			self.changeWheel = 0;
		elseif state == 1  then
			self.changeWheel = 1;
		elseif state == 2  then
			self.changeWheel = 2;
		end;
--______________________________________________________________________________
	elseif mode == 3 then
--______________________________________________________________________________
		local x,y,z = getTranslation(self.backAtt);
		if y<self.baTransMax then
			translate(self.backAtt, 0,0.001,0);
		end;
--______________________________________________________________________________
	elseif mode == 4 then
--______________________________________________________________________________
		local x,y,z = getTranslation(self.backAtt);
		if y>self.baTransMin then
			translate(self.backAtt, 0,-0.001,0);
		end;
--________________________________ Window _____________________________________
	elseif mode == 5 then
--______________________________________________________________________________
		if state2 then
			self:playAnimation(self.windowAnim, 1, nil, true);
			self.WM = true;
		else
			self:playAnimation(self.windowAnim, -1, nil, true);
			self.WM = false;
		end;
--________________________________ Door ______________________________________
	elseif mode == 6 then
--______________________________________________________________________________
		if state2 then
			if not self.WM then
				self:playAnimation(self.doorAnim, 1, nil, true);
				self.DM = true;
			else
				self:playAnimation(self.doorAnim, 1, nil, true);
				self.DM = true;
				self:playAnimation(self.windowAnim, -1, nil, true);
				self.WM = false;
			end;
		else
			self:playAnimation(self.doorAnim, -1, nil, true);
			self.DM = false;
		end;
--________________________________ Bonnet _______________________________________
	elseif mode == 7 then
--______________________________________________________________________________
		if state2 then
			self:playAnimation(self.bonnetAnim, 1, nil, true);
			self.BM = true;
		else
			self:playAnimation(self.bonnetAnim, -1, nil, true);
			self.BM = false;
		end;
--________________________________ Joystick ____________________________________
	elseif mode == 8 then
--______________________________________________________________________________
		setRotation(self.joystick, a1, a2, a3);
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:draw()
	
	if self.changeWheel == 0  then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLOP417_WHEEL_ALL"), self.typeDesc), InputBinding.MERLOP417_WHEEL_MODE);
	elseif self.changeWheel == 1  then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLOP417_WHEEL_CRAB"), self.typeDesc), InputBinding.MERLOP417_WHEEL_MODE);
	elseif self.changeWheel == 2  then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLOP417_WHEEL_FRONT"), self.typeDesc), InputBinding.MERLOP417_WHEEL_MODE);
	end;
	
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_BACK_ATTACHER_UP"), self.typeDesc), InputBinding.MERLO_BACK_ATTACHER_UP);
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_BACK_ATTACHER_DOWN"), self.typeDesc), InputBinding.MERLO_BACK_ATTACHER_DOWN);
	
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_DOOR_MODE"), self.typeDesc), InputBinding.MERLO_DOOR_MODE);
	if not self.DM then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("MERLO_WINDOW_MODE"), self.typeDesc), InputBinding.MERLO_WINDOW_MODE);
	end;
	
	if self.AC then
		self.ACOL:render();
		setTextBold(true);
		setTextColor(0.6307, 0.6307, 0.6307, 1);
		renderText(self.sX, self.sY, 0.018, "Service Reminder");
	end;
	
	if self.SC then
		self.SCOL:render();
		setTextBold(true);
		setTextColor(0.6307, 0.6307, 0.6307, 1);
		renderText(self.sX, self.sY, 0.018, "Engine failure!  Need service");
	end;
	
	if self.alarm then
		g_currentMission:addWarning("You can service the engine at the nearest vehicle selling point. Open the engine bonnet and turn off the engine first.", 0.07+0.022);
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:validateAttacherJoint(implement, jointDesc, dt)
    return true;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		self.changeWheel = Utils.getNoNil(getXMLInt(xmlFile, key.."#changeWheel"),0);
		self.WM = Utils.getNoNil(getXMLBool(xmlFile, key.."#WM"),false);
		self.DM = Utils.getNoNil(getXMLBool(xmlFile, key.."#DM"),false);
		self.BM = Utils.getNoNil(getXMLBool(xmlFile, key.."#BM"),false);
		self.AC = Utils.getNoNil(getXMLBool(xmlFile, key.."#AC"),false);
		self.SC = Utils.getNoNil(getXMLBool(xmlFile, key.."#SC"),false);
		
		self.nextService = Utils.getNoNil(getXMLFloat(xmlFile, key.."#nextService"),0);
		self.sumHours = Utils.getNoNil(getXMLFloat(xmlFile, key.."#totalHours"),0);
		
		self:MPCode(5, nil, nil, nil, nil, nil, self.WM);
		self:MPCode(6, nil, nil, nil, nil, nil, self.DM);
		self:MPCode(7, nil, nil, nil, nil, nil, self.BM);
		
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'changeWheel="'..tostring(self.changeWheel)..
	'" WM="'..tostring(self.WM)..
	'" DM="'..tostring(self.DM)..
	'" BM="'..tostring(self.BM)..
	'" AC="'..tostring(self.AC)..
	'" SC="'..tostring(self.SC)..
	'" nextService="'..tostring(self.nextService)..
	'" totalHours="'..tostring(self.sumHours)..'"';
	local nodes = nil;
	
	return attributes,nodes;
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:readStream(streamId, connection)
	self.changeWheel = streamReadInt32(streamId);
	self.WM = streamReadBool(streamId);
	self.DM = streamReadBool(streamId);
	self.BM = streamReadBool(streamId);
	self.AC = streamReadBool(streamId);
	self.SC = streamReadBool(streamId);
	self.nextService = streamReadFloat32(streamId);
	self.sumHours = streamReadFloat32(streamId);
	
	if self.changeWheel == 0  then
		self:MPCode(2, 0, 0, nil, nil, nil, nil, true);
	elseif self.changeWheel == 1  then
		self:MPCode(2, 1, -50, nil, nil, nil, nil, true);
	elseif self.changeWheel == 2  then
		self:MPCode(2, 2, 50, nil, nil, nil, nil, true);
	end;
	self:MPCode(5, nil, nil, nil, nil, nil, self.WM, true);
	self:MPCode(6, nil, nil, nil, nil, nil, self.DM, true);
	self:MPCode(7, nil, nil, nil, nil, nil, self.BM, true);
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:writeStream(streamId, connection)
	streamWriteInt32(streamId, self.changeWheel);
	streamWriteBool(streamId, self.WM);
	streamWriteBool(streamId, self.DM);
	streamWriteBool(streamId, self.BM);
	streamWriteBool(streamId, self.AC);
	streamWriteBool(streamId, self.SC);
	streamWriteFloat32(streamId, self.nextService);
	streamWriteFloat32(streamId, self.sumHours);
end;

--________________________________________________________________________________________________________________________________________________________
function MerloP417:playerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	
	local id = 0;
	for i,j in pairs(self.MTriggers) do
		if j.trigger == triggerId then
			id = i;
			break;
		end;
	end;
	
	if id ~= 0 then
		if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.MTriggers[id].enable = true;
		elseif onLeave then
			self.MTriggers[id].enable = false;
		end;
	end
	
end;

--**********************************************************************************************************************************************************************************
MerloP417Event = {};
MerloP417Event_mt = Class(MerloP417Event, Event);

InitEventClass(MerloP417Event, "MerloP417Event");

function MerloP417Event:emptyNew()
    local self = Event:new(MerloP417Event_mt);
    self.className="MerloP417Event";
    return self;
end;

function MerloP417Event:new(vehicle, mode, state, angle, a1, a2, a3, state2)
    local self = MerloP417Event:emptyNew()
    self.vehicle = vehicle;
	self.mode = mode;
	self.state = state;
	self.angle = angle;
	self.a1 = a1;
	self.a2 = a2;
	self.a3 = a3;
	self.state2 = state2;
    return self;
end;

function MerloP417Event:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.mode = streamReadInt32(streamId);
	self.state = streamReadInt32(streamId);
	self.angle = streamReadFloat32(streamId);
	self.a1 = streamReadFloat32(streamId);
	self.a2 = streamReadFloat32(streamId);
	self.a3 = streamReadFloat32(streamId);
	self.state2 = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function MerloP417Event:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, self.mode);
	streamWriteInt32(streamId, self.state);
	streamWriteFloat32(streamId, self.angle);
	streamWriteFloat32(streamId, self.a1);
	streamWriteFloat32(streamId, self.a2);
	streamWriteFloat32(streamId, self.a3);
	streamWriteBool(streamId, self.state2);
end;

function MerloP417Event:run(connection)
	self.vehicle:MPCode(self.mode, self.state, self.angle, self.a1, self.a2, self.a3, self.state2, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(MerloP417Event:new(self.vehicle, self.mode, self.state, self.angle, self.a1, self.a2, self.a3, self.state2), nil, connection, self.vehicle);
    end;
end;

function MerloP417Event.sendEvent(vehicle, mode, state, angle, a1, a2, a3, state2, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(MerloP417Event:new(vehicle, mode, state, angle, a1, a2, a3, state2), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(MerloP417Event:new(vehicle, mode, state, angle, a1, a2, a3, state2));
		end;
	end;
end;